<?php
/**
 * 2007-2019 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2019 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

use YaBandPay\PersiLiao\Cryptography;
use YaBandPay\PersiLiao\Notify;
use YaBandPay\PersiLiao\Payment;
use YaBandPay\PersiLiao\Response;
use YaBandPay\PersiLiao\Api;

/**
 * @project    : PrestaShop
 * @description:
 * @user       : PersiLiao
 * @email      : persi@sixsir.com
 * @date       : 2018/12/28
 * @time       : 22:24
 */
class YabandpayNotifyModuleFrontController extends ModuleFrontController
{
    /**
     * @var Yabandpay $module
     */
    public $module;
    
    public function postProcess()
    {
        try {
            $cryptography
                = new Cryptography($this->module->getYabandpayToken());
            $notify = new Notify($cryptography);
            $order_info = $notify->getOrderInfo();
            $order_state = Configuration::get('PS_OS_PAYMENT');
            if (isset($order_info['state'])
                && !empty($order_info['state'])
                && $order_info['state'] === Payment::PAY_PAID
            ) {
                //创建订单
                $this->module->validateOrder(
                    $order_info['demo']['cart_id'],
                    $order_state,
                    $order_info['demo']['cart_amount'],
                    $order_info['sub_pay_method'],
                    null,
                    array(),
                    $order_info['demo']['currency_id'],
                    false,
                    $order_info['demo']['secure_key']
                );
                $order_id = $this->module->currentOrder;
                Yabandpay::addYabandpayLogTradeId($order_id, $order_info['trade_id']);

                //向YabandPay发送修改order id请求
                $api = new Api(
                    new \YaBandPay\PersiLiao\Account(
                        $this->module->getYabandPayAccount(),
                        $this->module->getYabandPayToken()
                    ),
                    new \YaBandPay\PersiLiao\Request(
                        new Cryptography( $this->module->getYabandPayToken() )
                    )
                );
                $result = $api->updateOrderId($order_info['trade_id'], $order_id);      
            }
            Response::ajaxReturnSuccess();
        } catch (\Exception $e) {
            PrestaShopLogger::addLog(
                __CLASS__ . '::' . __FUNCTION__ . '->' . $e->getMessage()
            );
            Response::ajaxReturnFailure();
        }
    }
}
